/*
 * protocol_data.java
 * Douglas S. J. De Couto
 * 2 September 1999
 */

/*
 * Copyright (C) 1999 Douglas S. J. De Couto
 * <decouto@lcs.mit.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package decouto.garble;

/** Structure used to describe one capability of a device's interface
    protocol.  Used by application_layer.get_product_data.
    @author Douglas S. J. De Couto <a href="mailto:decouto@lcs.mit.edu">decouto@lcs.mit.edu</a>
    @version 2 September 1999 0.1 
    @see decouto.garble.application_layer */
class protocol_data {
  byte tag;
  int data;

  /** Tags used by the GPS to describe their protocol.  We don't use
      char here even though tags in spec are printable chars because
      it may change to use other kinds of byte values in future */
  static final byte 
  /** indicates a physical protocol */
  tag_phys_prot_id = (byte) 'P',
    /** indicates a link protocol */
    tag_link_prot_id = (byte) 'L',
    /** indicates an application protocol */
    tag_appl_prot_id = (byte) 'A',
    /** indicates a data type */
    tag_data_type_id = (byte) 'D';

  /** Returns a string describing this capability of the protocol.
      @return String with text describing the protocol type, fooled by
      the protocol version.  e.g. "link protocol 1" */ 
  public String toString() {
    // assume ascii encodings for chars
    if (tag == tag_phys_prot_id) {
      return "physical protocol " + data;
    }
    else if (tag == tag_link_prot_id) {
      return "link protocol " + data;
    }
    else if (tag == tag_appl_prot_id) {
      return "application protocol " + data;
    }
    else if (tag == tag_data_type_id) {
      return "data type " + data;
    }
    else {
      return "unknown tag; " + data;
    }
  }

}

