/*
 * util.java
 * Douglas S. J. De Couto
 * 29 August 1999
 */

/*
 * Copyright (C) 1999 Douglas S. J. De Couto
 * <decouto@lcs.mit.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package decouto.garble;

/** Utility routines used by the Garble pacakge.
    @author Douglas S. J. De Couto <a href="mailto:decouto@lcs.mit.edu">decouto@lcs.mit.edu</a>
    @version 29 August 1999 0.1 
    @see decouto.garble */
class util {

  /** Build a Java short whose value is the same as the two bytes
      stored in Garmin byte order starting at b[start].
      @return the short value whose MSB is in b[start + 1] and whose
      LSB is in b[start].
      @param b Array containing bytes of a short in Garmin byte order.
      @param start The offset of the Garmin format bytes in b. */
  static short bytes_to_short(byte b[], int start) {
    int i = ((b[start + 1] & 0xff) << 8) + (b[start] & 0xff);
    return (short) i;
  }
  /** Same as bytes_to_short(byte b[], int start).
      @return the short value whose MSB is in b[1] and whose
      LSB is in b[0].
      @param b Array whose first two bytes are a short in Garmin byte
      order. */
  static short bytes_to_short(byte b[]) {
    return bytes_to_short(b, 0);
  }
  
  /** Generate the 2 bytes in Garmin format representing the short s,
      and place them into b, MSB at b[start + 1], LSB at[start].
      @param s The short value to generate bytes for.
      @param b Array to contain generated bytes.
      @param start Offset in b at which to store the generated
      bytes. */
  static void short_to_bytes(short s, byte b[], int start) {
    b[start] = (byte) (s & 0xff);
    b[start + 1] = (byte) ((s & 0xff00) >> 8);
  }
  /** Same as short_to_bytes(short s, byte b[], int start).
      @param s The value to generate bytes for.
      @param b The array to contain the generated bytes in its first
      two elements. */
  static void short_to_bytes(short s, byte b[]) {
    short_to_bytes(s, b, 0);
  }
  /** Same as short_to_bytes(short s, byte b[])
      @return A fresh array of size 2 containing the generated bytes. 
      @param s The value to generate bytes for. */
  static byte[] short_to_bytes(short s) {
    byte b[] = new byte[2];
    short_to_bytes(s, b, 0);
    return b;
  }

  /** Analagous to bytes_to_short.
      @see decouto.garble.util.short_to_int */
  static int bytes_to_int(byte b[], int start) {
    /* return the int value whose MSB is in b[start + 3] and whose
       LSB is in b[start]. */
   int i = ((b[start + 3] & 0xff) << 24) + ((b[start + 2] & 0xff) << 16)
     + ((b[start + 1] & 0xff) << 8) + (b[start] & 0xff);
   return i;
  }
  /** Analagous to bytes_to_short.
      @see decouto.garble.util.bytes_to_short */
  static int bytes_to_int(byte b[]) {
    /* as above, but start == 0 */
    return bytes_to_int(b, 0);
  }
  
  /** Analagous to short_to_bytes.
      @see decouto.garble.util.short_to_bytes */
  static void int_to_bytes(int i, byte b[], int start) {
    /* store the bytes of int i into b, MSB into b[start + 3] and
       LSB into b[start] */
    b[start] = (byte) (i & 0xff);
    b[start + 1] = (byte) ((i & 0xff00) >> 8);
    b[start + 2] = (byte) ((i & 0xff0000) >> 16);
    b[start + 3] = (byte) ((i & 0xff000000) >> 24);
  }
  /** Analagous to short_to_bytes.
      @see decouto.garble.util.short_to_bytes */
  static void int_to_bytes(int i, byte b[]) {
    /* as above, but start == 0 */
    int_to_bytes(i, b, 0);
  }
  /** Analagous to short_to_bytes.
      @see decouto.garble.util.short_to_bytes */
  static byte[] int_to_bytes(int i) {
    /* return an array b of two bytes with MSB of s in b[1] and LSB in
       b[0] */
    byte b[] = new byte[4];
    int_to_bytes(i, b, 0);
    return b;
  }

}
